/***************************************************************************
                          names.cpp  -  RPG names
                             -------------------
    begin                : 10.10.2002
    copyright            : (C) 2002 by strato
    email                : strato@shufflebee.dk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "names.h"
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qtabwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qmessagebox.h>
#include <qfile.h>
//#include <qpe/fileselector.h>
#include <qpe/config.h>
#include <qxml.h>
#include <qfont.h>

/* 
 *  Constructs a GMTool which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
GMTool::GMTool( QWidget* parent,  const char* name, WFlags fl )
    : BaseForm( parent, name, fl )
{
  TabGroup->setCurrentPage(0);  

  qDebug("executing RPG Names");
  QFont f("Helvetica", 10, QFont::Normal, FALSE, QFont::ISO_8859_4);

  this->setFont( f );  
  //fs = new FileSelector("text/xml", this, "XML Browser", false);
  //connect( fs, SIGNAL(fileSelected(const DocLnk&)), this, SLOT(openDoc(const DocLnk&))); 
}

/*  
 *  Destroys the object and frees any allocated resources
 */
GMTool::~GMTool()
{
  // no need to delete child widgets, Qt does it all for us
  // Todo: move file Config here...!
}

/*
 * 
 */
void GMTool::setDocument(const QString& fileObj)
{  
  setFile( fileObj );
  parseFile();
}

/*
 * 
 */
void GMTool::setFile(QString file)
{
  //qDebug("try: xmlFile configuration setting file = " + file);
  Config cfg = Config("rpgnames");
  cfg.setGroup("xmlfiles");
  cfg.writeEntry("filename",file);
  xmlFile.setName(file);
  //qDebug("done: xmlFile configuration setting file = " + file);
}

/*
 * Responsible for handling the XML file
 * given as input to the application.
 */
void GMTool::parseFile()
{
  handler.clear();
  QXmlInputSource source( xmlFile );
  reader.setContentHandler( &handler);      
  reader.parse( source );

  ComboBoxRaces->clear();  
  ComboBoxPlaces->clear();
  ComboBoxGeo->clear();

  ComboBoxGeo->insertItem("None");
  ComboBoxPlaces->insertItem("None");

  RandomPlaceList::Iterator it;
  RandomNameList::Iterator pit;

  RandomNameList a = handler.races;
  for( pit = a.begin(); pit != a.end(); ++pit )	   
     ComboBoxRaces->insertItem((*pit).uname);

  RandomPlaceList g = handler.geoplaces;
  for( it = g.begin(); it != g.end(); it++ )
    ComboBoxGeo->insertItem((*it).placename);

  RandomPlaceList p = handler.places;
  for( it = p.begin(); it != p.end(); it++ )
    ComboBoxPlaces->insertItem((*it).placename);
}

/*
 * public slot for selecting XML file
 */
void GMTool::openXML()
{
  //fs = new FileSelector("text/xml", this, "fileselector", false);
  //connect( fs, SIGNAL(fileSelected(const DocLnk&)), this, SLOT(openDoc(const DocLnk&)));  
  // this.show(fs);  
  qDebug("Missing feature");
}

void GMTool::openDoc( const DocLnk& xmlfile)
{
  setFile(xmlfile.file());
  parseFile();  
}

/*
 * public slot for selecting between geo and places
 */
void GMTool::geoChange()
{
  if(ComboBoxGeo->currentItem() != 0)
    ComboBoxPlaces->setCurrentItem(0);
}

/*
 * public slot for selecting between geo and places
 */
void GMTool::placeChange()
{
  if(ComboBoxPlaces->currentItem() != 0)
    ComboBoxGeo->setCurrentItem(0);
}


/* 
 * public slot
 */
void GMTool::quit()
{
    close(); 
}

/*
 * public slot for rolling random NPCs
 */
void GMTool::rollNPC()
{
  QMessageBox::information(this,"RPG names","NPC generator not implemented yet!");
  qWarning("NPC feature not generated yet ....");
}

/*
 * public slot generate: delegates the different generations
 * according to current Tab page...
 */
void GMTool::generate()
{
  switch(TabGroup->currentPageIndex())
    {
    case 0: rollNames(); break;
    case 1: rollPlaces(); break;
    case 2: rollNPC(); break;
    default: QMessageBox::information(this,"RPG names","Error: no known Tab page chosen"); break;
    }
}


/* 
 * public slot
 */
void GMTool::rollPlaces()
{
  int geo, pla;
  geo = ComboBoxGeo->currentItem();
  pla = ComboBoxPlaces->currentItem();

  // Nothing chosen
  if(geo == 0 && pla == 0)
    {
      QMessageBox::information(this,"RPG names","Please choose something!");
      return;
    }
 
  int max,r1,r2,r3,r4,r5;
  
  unsigned seed = time(NULL);
  srand(seed);

  RandomPlaceList::Iterator it;
  
  QString place;
  if( geo == 0) 
    {
      place = ComboBoxPlaces->currentText();
      for( it = handler.places.begin(); it != handler.places.end(); ++it )
	{

	  if( place == (*it).placename )
	    {
	      max = (*it).p1.count();
	      r1 = rand()%max;
	      r2 = rand()%max;
	      r3 = rand()%max;
	      r4 = rand()%max;
	      r5 = rand()%max;

	      TextLabelPlace1->setText((*it).p1[r1]);
	      TextLabelPlace2->setText((*it).p2[r2]);
	      TextLabelPlace3->setText((*it).p3[r3]);
	      TextLabelPlace4->setText((*it).p4[r4]);
	      TextLabelPlace5->setText((*it).p5[r5]);
	      return;
	    }
	}
    }
  else 
    {
      place = ComboBoxGeo->currentText();
      for( it = handler.geoplaces.begin(); it != handler.geoplaces.end(); ++it )
	{
	  if( place == (*it).placename )
	    {
	      max = (*it).p1.count();
	      r1 = rand()%max;
	      r2 = rand()%max;
	      r3 = rand()%max;
	      r4 = rand()%max;
	      r5 = rand()%max;

	      TextLabelPlace1->setText((*it).p1[r1]);
	      TextLabelPlace2->setText((*it).p2[r2]);
	      TextLabelPlace3->setText((*it).p3[r3]);
	      TextLabelPlace4->setText((*it).p4[r4]);
	      TextLabelPlace5->setText((*it).p5[r5]);
	      return;
	    }
	}
    }
}

/* 
 * public slot
 */
void GMTool::rollNames()
{
  int gender,max,r1,r2,r3,r4,r5;
  
  unsigned seed = time(NULL);
  srand(seed);
  
  QString race;
  race  = ComboBoxRaces->currentText();
  if(RadioButtonFemale->isChecked()) gender = 0; else gender = 1;

  // LagerSegment fejl on this one:
  //QFont f2("Helvetica", 9, QFont::Normal, FALSE, QFont::ISO_8859_1);
  //this->setFont( f2 );  
  
  RandomNameList::Iterator it;
  for( it = handler.races.begin(); it != handler.races.end(); ++it )	
    { 
      if( race == (*it).uname )
	{
	  if( gender == 1) 
	    {	      
	      max = (*it).m1.count();
	      r1 = rand()%max;
	      r2 = rand()%max;
	      r3 = rand()%max;
	      r4 = rand()%max;
	      r5 = rand()%max;

	     TextLabelName1->setText((*it).m1[r1]);
	     TextLabelName2->setText((*it).m2[r2]);
	     TextLabelName3->setText((*it).m3[r3]);
	     TextLabelName4->setText((*it).m4[r4]);
	     TextLabelName5->setText((*it).m5[r5]);
	     break;
	    }
	  else
	    {
	      max = (*it).f1.count();
	      r1 = rand()%max;
	      r2 = rand()%max;
	      r3 = rand()%max;
	      r4 = rand()%max;
	      r5 = rand()%max;

	     TextLabelName1->setText((*it).f1[r1]);
	     TextLabelName2->setText((*it).f2[r2]);
	     TextLabelName3->setText((*it).f3[r3]);
	     TextLabelName4->setText((*it).f4[r4]);
	     TextLabelName5->setText((*it).f5[r5]);
	     break;
	    }	  
	}
    }  
}

