/***************************************************************************
                          xmlparser.cpp  -  RPG names
                             -------------------
    begin                : 10.10.2002
    copyright            : (C) 2002 by strato
    email                : strato@shufflebee.dk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xmlparser.h"
#include <qstring.h>

const int BEGINDOC    = 1;
const int ENDDOC      = 0;
const int BEGINRACES  = 2;
const int ENDRACES    = 3;
const int BEGINPLACES = 4;
const int ENDPLACES   = 5;
const int BEGINGEO    = 6;
const int ENDGEO      = 7;

const int BEGINRACE   = 10;
const int ENDRACE     = 11;
const int BEGINMALE   = 12;
const int ENDMALE     = 13;
const int BEGINFEMALE = 14;
const int ENDFEMALE   = 15;

bool XmlParser::startDocument()
{  
  state = BEGINDOC; 
  return TRUE;
}

bool XmlParser::endDocument()
{  
  if(!races.isEmpty())
    {    

    }
  return TRUE;
}

bool XmlParser::startElement( const QString&, const QString&, 
			      const QString& qName, const QXmlAttributes& attributes )
{
  if(qName == "rpgnames") state = BEGINDOC; 
  if(qName == "races") state = BEGINRACES;
  if(qName == "race") state = BEGINRACE;
  if(qName == "male") { state = BEGINMALE; gender = 1; }
  if(qName == "female") { state = BEGINFEMALE; gender = 0; } 
  if(qName == "places") state = BEGINPLACES; 
  if(qName == "geo") state = BEGINGEO; 
 
  switch(state)
    {
    case BEGINRACE: racename = attributes.value("value"); females.clear(); males.clear(); break;    
    case BEGINPLACES: placename = attributes.value("value"); place.clear(); break;
    case BEGINGEO: placename = attributes.value("value"); place.clear(); break;
    default: break;  
    }
 
  return TRUE;
}

bool XmlParser::endElement( const QString&, const QString&, const QString& qName )
{
  switch(state)
    {
    case BEGINDOC: if(qName == "rpgnames") state = ENDDOC; break;
    case BEGINRACE: if(qName == "race") state = ENDRACE; break;
    case BEGINMALE: if(qName == "male") state = ENDMALE; break;
    case BEGINFEMALE: if(qName == "female") state = ENDFEMALE; break;   
    case ENDFEMALE: if(qName == "race") state = ENDRACE; break;      
    case BEGINPLACES: if(qName == "places") state = ENDPLACES; break;
    case BEGINGEO: if(qName == "geo") state = ENDGEO; break;
    default: state = ENDDOC; break;  
    }
 
  if(state == ENDFEMALE)
    {
      RUnit ru; 
      ru.uname = racename;
      ru.makeLists(females,males);
      races.append(ru);
    }

  if(state == ENDPLACES)
    {
      PUnit pu;
      pu.placename = placename;
      pu.makeList(place);
      places.append(pu);
    }

  if(state == ENDGEO)
    {
      PUnit gu;
      gu.placename = placename;
      gu.makeList(place);
      geoplaces.append(gu);
    }
  
  return TRUE;
}

bool XmlParser::characters( const QString& data )
{  
  //QString datas;
  //datas = datas.setLatin1(data);

  switch(state) 
    {
    case BEGINMALE:
      {
	males  = males.split(",",data);  
	trimList( males );
	break;
      }
    case BEGINFEMALE:
      {
	females = females.split(",",data);
	trimList( females );
	break;
      }
    case BEGINPLACES:
      {
	place = place.split(",",data);
	trimList( place );
	break;
      }
    case BEGINGEO:
      {
	place = place.split(",",data);
	trimList( place );
	break;
      }
    }
  return TRUE;
}

void XmlParser::clear()
{
  races.clear();
  places.clear();
  geoplaces.clear();  
}

void XmlParser::trimList( QStringList& data )
{
  for( QStringList::Iterator it = data.begin(); it != data.end(); ++it ) 
    {
      (*it) = (*it).simplifyWhiteSpace();
    } 
}
